@@ -21,16 +21,16 @@
  *  Copyright 2003, 2004, 2005 Øyvind Kolås <pippin@gimp.org>
  */
 
+#define _POSIX_C_SOURCE 200112L
+
 #include "config.h"
 #include <math.h>
 #include <string.h>
+#include <stdint.h>
 
 #include "babl.h"
 #include "extensions/util.h"
 
-
-#define INLINE    inline
-
 /*
  * Implemented according to information read from:
  *
@@ -59,6 +59,7 @@ static float          table_16_F[1 << 16];
 static unsigned char  table_F_8[1 << 16];
 static unsigned short table_F_16[1 << 16];
 
+static uint32_t      *table_8_F_int = NULL;
 
 static int table_inited = 0;
 
@@ -67,6 +68,9 @@ table_init (void)
 {
   if (table_inited)
     return;
+  
+  table_8_F_int = (void*)(table_8_F);
+
   table_inited = 1;
 
   /* fill tables for conversion from integer to float */
@@ -88,7 +92,7 @@ table_init (void)
     } u;
     u.f = 0.0;
 
-    u.s[0] = 0.0;
+    u.s[0] = 0x8000;
 
     for (u.s[1] = 0; u.s[1] < 65535; u.s[1] += 1)
       {
@@ -107,8 +111,8 @@ table_init (void)
           }
         else
           {
-            c = rint (u.f * 255.0);
-            s = rint (u.f * 65535.0);
+            c = lrint (u.f * 255.0);
+            s = lrint (u.f * 65535.0);
           }
 
         /*fprintf (stderr, "%2.3f=%03i %05i ", f, c, (*hi));
@@ -147,7 +151,7 @@ gggl_float_to_index16 (float f)
   return u.s[1];
 }
 
-static INLINE long
+static long
 conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -164,7 +168,7 @@ conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -181,7 +185,7 @@ conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_8_F (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -190,14 +194,38 @@ conv_8_F (unsigned char *src, unsigned char *dst, long samples)
     table_init ();
   while (n--)
     {
-      (*(float *) dst) = table_8_F[*(unsigned char *) src];
+      (*(uint32_t *) dst) = table_8_F_int[*(unsigned char *) src];
       dst             += 4;
       src             += 1;
     }
   return samples;
 }
 
-static INLINE long
+static long
+conv_rgb8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  if (!table_inited)
+    table_init ();
+  while (n--)
+    {
+      (*(uint32_t *) dst) = table_8_F_int[*(unsigned char *) src];
+      dst             += 4;
+      src             += 1;
+      (*(uint32_t *) dst) = table_8_F_int[*(unsigned char *) src];
+      dst             += 4;
+      src             += 1;
+      (*(uint32_t *) dst) = table_8_F_int[*(unsigned char *) src];
+      dst             += 4;
+      src             += 1;
+      (*(float    *) dst) = 1.0;
+      dst             += 4;
+    }
+  return samples;
+}
+
+static long
 conv_16_F (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -213,143 +241,7 @@ conv_16_F (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-#else
-
-static INLINE long
-conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      float f    = ((*(float *) src));
-      int   uval = rint (f * 255.0);
-
-      if (uval < 0) uval = 0;
-      if (uval > 255) uval = 255;
-      *(unsigned char *) dst = uval;
-
-      dst += 1;
-      src += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      float f = ((*(float *) src));
-      if (f < 0.0)
-        {
-          *(unsigned short *) dst = 0;
-        }
-      else if (f > 1.0)
-        {
-          *(unsigned short *) dst = 65535;
-        }
-      else
-        {
-          *(unsigned short *) dst = rint (f * 65535.0);
-        }
-      dst += 2;
-      src += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_8_F (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      (*(float *) dst) = ((*(unsigned char *) src) / 255.0);
-      dst             += 4;
-      src             += 1;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_16_F (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      (*(float *) dst) = *(unsigned short *) src / 65535.0;
-      dst             += 4;
-      src             += 2;
-    }
-  return samples;
-}
-
-
-#endif
-
-
-static INLINE long
-conv_F_D (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      *(double *) dst = ((*(float *) src));
-      dst            += 8;
-      src            += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_D_F (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      *(float *) dst = ((*(double *) src));
-      dst           += 4;
-      src           += 8;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_16_8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      (*(unsigned char *) dst) = (*(unsigned short *) src) >> 8;
-      dst                     += 1;
-      src                     += 2;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_8_16 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      (*(unsigned short *) dst) = (*(unsigned char *) src) << 8;
-      dst                      += 2;
-      src                      += 1;
-    }
-  return samples;
-}
-
-static INLINE long
+static long
 conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -376,25 +268,185 @@ conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
+#else
+
+static long
+conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float f    = ((*(float *) src));
+      int   uval = lrint (f * 255.0);
+
+      if (uval < 0) uval = 0;
+      if (uval > 255) uval = 255;
+      *(unsigned char *) dst = uval;
+
+      dst += 1;
+      src += 4;
+    }
+  return samples;
+}
+
+static long
+conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float f = ((*(float *) src));
+      if (f < 0.0)
+        {
+          *(unsigned short *) dst = 0;
+        }
+      else if (f > 1.0)
+        {
+          *(unsigned short *) dst = 65535;
+        }
+      else
+        {
+          *(unsigned short *) dst = lrint (f * 65535.0);
+        }
+      dst += 2;
+      src += 4;
+    }
+  return samples;
+}
+
+static long
+conv_8_F (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      (*(float *) dst) = ((*(unsigned char *) src) / 255.0);
+      dst             += 4;
+      src             += 1;
+    }
+  return samples;
+}
+
+static long
+conv_16_F (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      (*(float *) dst) = *(unsigned short *) src / 65535.0;
+      dst             += 4;
+      src             += 2;
+    }
+  return samples;
+}
+
+static long
+conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      int c;
+
+      for (c = 0; c < 3; c++)
+        {
+          int val = rint ((*(float *) src) * 255.0);
+          if (val < 0)
+            *(unsigned char *) dst = 0;
+          else if (val > 255)
+            *(unsigned char *) dst = 255;
+          else
+            *(unsigned char *) dst = val;
+          dst += 1;
+          src += 4;
+        }
+      src += 4;
+    }
+  return samples;
+}
+
+#endif
+
+
+static long
+conv_F_D (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      *(double *) dst = ((*(float *) src));
+      dst            += 8;
+      src            += 4;
+    }
+  return samples;
+}
+
+static long
+conv_D_F (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      *(float *) dst = ((*(double *) src));
+      dst           += 4;
+      src           += 8;
+    }
+  return samples;
+}
+
+static long
+conv_16_8 (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+#define div_257(a) ((((a)+128)-(((a)+128)>>8))>>8)
+      (*(unsigned char *) dst) = div_257 (*(unsigned short *) src);
+      dst                     += 1;
+      src                     += 2;
+    }
+  return samples;
+}
+
+static long
+conv_8_16 (unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      (*(unsigned short *) dst) = ((*(unsigned char *) src) << 8) | *src;
+      dst                      += 2;
+      src                      += 1;
+    }
+  return samples;
+}
+
 
 /*********/
-static INLINE long
+static long
 conv_rgbaF_rgba8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_8 (src, dst, samples * 4);
   return samples;
 }
 
-#define conv_rgbaF_rgbP8    conv_rgbaF_rgba8
-
-static INLINE long
+static long
 conv_rgbF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_8 (src, dst, samples * 3);
   return samples;
 }
 
-static INLINE long
+static long
 conv_gaF_ga8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_8 (src, dst, samples * 2);
@@ -405,21 +457,21 @@ conv_gaF_ga8 (unsigned char *src, unsigned char *dst, long samples)
 #define conv_gF_g8          conv_F_8
 #define conv_gAF_gA8        conv_gaF_ga8
 
-static INLINE long
+static long
 conv_rgbaF_rgba16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_16 (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgbF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_16 (src, dst, samples * 3);
   return samples;
 }
 
-static INLINE long
+static long
 conv_gaF_ga16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_16 (src, dst, samples * 2);
@@ -430,19 +482,20 @@ conv_gaF_ga16 (unsigned char *src, unsigned char *dst, long samples)
 #define conv_gF_g16          conv_F_16
 #define conv_gAF_gA16        conv_gaF_ga16
 
-static INLINE long
+static long
 conv_rgba8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   return conv_8_F (src, dst, samples * 4) / 4;
 }
 
-static INLINE long
+
+static long
 conv_rgb8_rgbF (unsigned char *src, unsigned char *dst, long samples)
 {
   return conv_8_F (src, dst, samples * 3) / 3;
 }
 
-static INLINE long
+static long
 conv_ga8_gaF (unsigned char *src, unsigned char *dst, long samples)
 {
   return conv_8_F (src, dst, samples * 2) / 2;
@@ -452,35 +505,35 @@ conv_ga8_gaF (unsigned char *src, unsigned char *dst, long samples)
 #define conv_gA8_gAF        conv_ga8_gaF
 #define conv_g8_gF          conv_8_F
 
-static INLINE long
+static long
 conv_rgbaF_rgbaD (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_F_D (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgbaD_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_D_F (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgba16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_F (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgb16_rgbF (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_F (src, dst, samples * 3);
   return samples;
 }
 
-static INLINE long
+static long
 conv_ga16_gaF (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_F (src, dst, samples * 2);
@@ -491,21 +544,21 @@ conv_ga16_gaF (unsigned char *src, unsigned char *dst, long samples)
 #define conv_gA16_gAF        conv_ga16_gaF
 #define conv_g16_gF          conv_16_F
 
-static INLINE long
+static long
 conv_rgba16_rgba8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_8 (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgb16_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_8 (src, dst, samples * 3);
   return samples;
 }
 
-static INLINE long
+static long
 conv_ga16_ga8 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_16_8 (src, dst, samples * 2);
@@ -516,21 +569,21 @@ conv_ga16_ga8 (unsigned char *src, unsigned char *dst, long samples)
 #define conv_gA16_gA8        conv_ga16_ga8
 #define conv_g16_g8          conv_16_8
 
-static INLINE long
+static long
 conv_rgba8_rgba16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_8_16 (src, dst, samples * 4);
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgb8_rgb16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_8_16 (src, dst, samples * 3);
   return samples;
 }
 
-static INLINE long
+static long
 conv_ga8_ga16 (unsigned char *src, unsigned char *dst, long samples)
 {
   conv_8_16 (src, dst, samples * 2);
@@ -543,7 +596,7 @@ conv_ga8_ga16 (unsigned char *src, unsigned char *dst, long samples)
 
 /* alpha conversions */
 
-static INLINE long
+static long
 conv_gaF_gAF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -562,7 +615,7 @@ conv_gaF_gAF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_gAF_gaF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -581,111 +634,22 @@ conv_gAF_gaF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
-conv_rgbAF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      float alpha = (((float *) src)[3]);
-      int   c;
-      if (alpha >= 1.0)
-        {
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = *(float *) src;
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      else if (alpha <= 0.0)
-        {
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = 0;
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      else
-        {
-          float alpha_recip = 1.0 / alpha;
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = ((*(float *) src) * alpha_recip);
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      *(float *) dst = alpha;
-      dst           += 4;
-      src           += 4;
-    }
-  return samples;
-}
-
-
-static INLINE long
-conv_rgbaF_rgbAF (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      float alpha = (((float *) src)[3]);
-      int   c;
-
-      if (alpha >= 1.0)
-        {
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = *(float *) src;
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      else if (alpha <= 0.0)
-        {
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = 0;
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      else
-        {
-          for (c = 0; c < 3; c++)
-            {
-              *(float *) dst = ((*(float *) src) * alpha);
-              dst           += 4;
-              src           += 4;
-            }
-        }
-      *(float *) dst = alpha;
-      dst           += 4;
-      src           += 4;
-    }
-  return samples;
-}
-
 /* alpha stripping and adding */
 
-static INLINE long
+static long
 conv_rgbaF_rgbF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
   while (n--)
     {
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       dst           += 4;
       src           += 4;
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       dst           += 4;
       src           += 4;
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       dst           += 4;
       src           += 4;
       src           += 4;
@@ -693,29 +657,32 @@ conv_rgbaF_rgbF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgbF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
   while (n--)
     {
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       src           += 4;
       dst           += 4;
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       src           += 4;
       dst           += 4;
-      *(float *) dst = (*(float *) src);
+      *(uint32_t *) dst = (*(uint32_t *) src);
       src           += 4;
       dst           += 4;
-      *(float *) dst = 1.0;
+      *(uint32_t *) dst = 1.0;
       dst           += 4;
     }
   return samples;
 }
 
-static INLINE long
+#define conv_rgbF_rgbAF    conv_rgbF_rgbaF
+
+
+static long
 conv_gaF_gF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -730,7 +697,7 @@ conv_gaF_gF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_gF_gaF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -752,7 +719,7 @@ conv_gF_gaF (unsigned char *src, unsigned char *dst, long samples)
 
 /* colorchannel dropping and adding */
 
-static INLINE long
+static long
 conv_gF_rgbF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -771,7 +738,7 @@ conv_gF_rgbF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_gaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -798,7 +765,7 @@ conv_gaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 /* other conversions coded for some optimisation reason or sumthin */
 
 
-static INLINE long
+static long
 conv_rgbaF_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -810,68 +777,18 @@ conv_rgbaF_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
 
       for (c = 0; c < 3; c++)
         {
-          *(unsigned char *) dst = ((*(float *) src) * alpha) * 255.0;
+          *(unsigned char *) dst = lrint (((*(float *) src) * alpha) * 255.0);
           dst                   += 1;
           src                   += 4;
         }
-      *(unsigned char *) dst = alpha * 255.0;
+      *(unsigned char *) dst = lrint (alpha * 255.0);
       dst++;
       src += 4;
     }
   return samples;
 }
 
-static INLINE long
-conv_rgbaF_rgbA16 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      float alpha = (*(float *) src + (4 * 3));
-      int   c;
-
-      for (c = 0; c < 3; c++)
-        {
-          *(unsigned short *) dst = ((*(float *) src) * alpha) * 65535.0;
-          dst                    += 2;
-          src                    += 4;
-        }
-      *(unsigned short *) dst = alpha * 65535.0;
-      dst                    += 2;
-      src                    += 4;
-    }
-  return samples;
-}
-#if 0
-static INLINE long
-conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      int c;
-
-      for (c = 0; c < 3; c++)
-        {
-          int val = rint ((*(float *) src) * 255.0);
-          if (val < 0)
-            *(unsigned char *) dst = 0;
-          else if (val > 255)
-            *(unsigned char *) dst = 255;
-          else
-            *(unsigned char *) dst = val;
-          dst += 1;
-          src += 4;
-        }
-      src += 4;
-    }
-  return samples;
-}
-#endif
-
-static INLINE long
+static long
 conv_rgbaF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -882,7 +799,7 @@ conv_rgbaF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
 
       for (c = 0; c < 3; c++)
         {
-          *(unsigned short *) dst = (*(float *) src) * 65535.0;
+          *(unsigned short *) dst = lrint ((*(float *) src) * 65535.0);
           dst                    += 2;
           src                    += 4;
         }
@@ -891,29 +808,7 @@ conv_rgbaF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
-conv_rgbP8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      int c;
-
-      for (c = 0; c < 3; c++)
-        {
-          (*(float *) dst) = *(unsigned char *) src / 255.0;
-          dst             += 4;
-          src             += 1;
-        }
-      (*(float *) dst) = 1.0;
-      dst             += 4;
-      src             += 1;
-    }
-  return samples;
-}
-
-static INLINE long
+static long
 conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -937,7 +832,7 @@ conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 }
 
 /*
-   static INLINE long
+   static long
    conv_rgb8_rgbaF (unsigned char *src,
                  unsigned char *dst,
                  int samples)
@@ -957,7 +852,7 @@ conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
    return samples;
    }
 
-   static INLINE long
+   static long
    conv_g8_rgbaF (unsigned char *src,
                unsigned char *dst,
                int samples)
@@ -977,7 +872,7 @@ conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
    return samples;
    }
 
-   static INLINE long
+   static long
    conv_rgb16_rgbaF (unsigned char *src,
                   unsigned char *dst,
                   int samples)
@@ -998,7 +893,7 @@ conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
    return samples;
    }
 
-   static INLINE long
+   static long
    conv_gF_rgbaF (unsigned char *src,
                unsigned char *dst,
                int samples)
@@ -1019,7 +914,7 @@ conv_rgbA16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
    return samples;
    }
  */
-static INLINE long
+static long
 conv_rgba8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -1036,9 +931,10 @@ conv_rgba8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
         }
       else
         {
-          dst[0] = (src[0] * src[3]) >> 8;      /* FIXME: check if this faster */
-          dst[1] = (src[1] * src[3]) >> 8;      /*        version (>>8 vs /255) */
-          dst[2] = (src[2] * src[3]) >> 8;      /*        is accurate enough   */
+#define div_255(a) ((((a)+127)+(((a)+127)>>8))>>8)
+          dst[0] = div_255 (src[0] * src[3]);
+          dst[1] = div_255 (src[1] * src[3]);
+          dst[2] = div_255 (src[2] * src[3]);
           dst[3] = src[3];
         }
       dst += 4;
@@ -1047,7 +943,7 @@ conv_rgba8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgbA8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -1066,10 +962,10 @@ conv_rgbA8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
         }
       else
         {
-          unsigned int aa = (255 << 16) / src[3];
-          *dst++ = (src[0] * aa) >> 16;
-          *dst++ = (src[1] * aa) >> 16;
-          *dst++ = (src[2] * aa) >> 16;
+          unsigned int aa = ((255 << 16) + (src[3] >> 1)) / src[3];
+          *dst++ = (src[0] * aa + 0x8000) >> 16;
+          *dst++ = (src[1] * aa + 0x8000) >> 16;
+          *dst++ = (src[2] * aa + 0x8000) >> 16;
           *dst++ = src[3];
         }
       src += 4;
@@ -1077,225 +973,26 @@ conv_rgbA8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
-conv_argb8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      dst[3] = src[3];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_rgba8_argb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[3];
-      dst[1] = src[0];
-      dst[2] = src[1];
-      dst[3] = src[2];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_Argb8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      dst[3] = src[3];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_rgbA8_Argb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      dst[3] = src[3];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_Prgb8_rgbP8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      dst[3] = 255;
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_rgbP8_Prgb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      dst[3] = 255;
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_Prgb8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];
-      dst[1] = src[1];
-      dst[2] = src[0];
-      /*dst[3] = src[3]; */
-      src += 4;
-      dst += 3;
-    }
-  return samples;
-}
-
-
-/* FIXME: this is actually BGRP */
-static INLINE long
-conv_rgb8_Prgb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[2];          /* blue */
-      dst[1] = src[1];          /* green */
-      dst[2] = src[0];          /* red */
-      dst[3] = 255;
-      src   += 3;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_rgb8_rgbP8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[0];
-      dst[1] = src[1];
-      dst[2] = src[2];
-      src   += 3;
-      dst   += 4;
-    }
-  return samples;
-}
-
-static INLINE long
+static long
 conv_rgb8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
 {
-  long n = samples;
-
+  long n = samples-1;
   while (n--)
     {
-      /**(unsigned int *) dst = *(unsigned int *) src;
-         dst[3] = 255;*/
-
-      dst[0] = src[0];
-      dst[1] = src[1];
-      dst[2] = src[2];
-      dst[3] = 255;
+      *(unsigned int *) dst = (*(unsigned int *) src) | (255 << 24);
       src   += 3;
       dst   += 4;
     }
+  dst[0] = src[0];
+  dst[1] = src[1];
+  dst[2] = src[2];
+  dst[3] = 255;
   return samples;
 }
 
 #define conv_rgb8_rgbA8    conv_rgb8_rgba8
 
-static INLINE long
-conv_rgbP8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      *(unsigned int *) dst = *(unsigned int *) src;
-      dst[3]                = 255;
-      src                  += 4;
-      dst                  += 4;
-    }
-  return samples;
-}
-
-#define conv_rgbP8_rgbA8    conv_rgbP8_rgba8
-
-static INLINE long
-conv_rgba8_rgbP8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  memcpy (dst, src, samples * 4);
-  return samples;
-}
-
-#define conv_rgbA8_rgbP8    conv_rgba8_rgbP8
-
-static INLINE long
-conv_rgbP8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
-{
-  long n = samples;
-
-  while (n--)
-    {
-      dst[0] = src[0];
-      dst[1] = src[1];
-      dst[2] = src[2];
-      src   += 4;
-      dst   += 3;
-    }
-  return samples;
-}
-
-static INLINE long
+static long
 conv_rgba8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -1311,7 +1008,7 @@ conv_rgba8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static INLINE long
+static long
 conv_rgbA8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
@@ -1333,10 +1030,10 @@ conv_rgbA8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
         }
       else
         {
-          unsigned int aa = (255 << 16) / alpha;
-          *dst++ = (src[0] * aa) >> 16;
-          *dst++ = (src[1] * aa) >> 16;
-          *dst++ = (src[2] * aa) >> 16;
+          unsigned int aa = ((255 << 16) + (alpha >> 1)) / alpha;
+          *dst++ = (src[0] * aa + 0x8000) >> 16;
+          *dst++ = (src[1] * aa + 0x8000) >> 16;
+          *dst++ = (src[2] * aa + 0x8000) >> 16;
         }
       src += 4;
     }
@@ -1367,7 +1064,7 @@ conv_rgbA8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
 
 
 
-  static INLINE long
+  static long
 conv_yuvaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 {
   float *src_f = (float *) src;
@@ -1399,7 +1096,7 @@ conv_yuvaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
 }
 
 
-static INLINE long
+static long
 conv_yuvF_rgbF (unsigned char *src, unsigned char *dst, long samples)
 {
   float *src_f = (float *) src;
@@ -1429,203 +1126,6 @@ conv_yuvF_rgbF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-
-/******* lab, xyz and rgb interaction, lifted from cpercep by adam d. moss */
-
-/*
-   static const double Xn = 0.951;
-   static const double Yn = 1.0;
-   static const double Zn = 1.089;
- */
-
-#define LXN    0.312713F
-#define LYN    0.329016F
-
-static const double lxn = LXN;
-static const double lyn = LYN;
-static double       xnn = LXN / LYN;
-static double       znn = (1.0F - (LXN + LYN)) / LYN;
-
-
-static const double LRAMP = 7.99959199F;
-
-static INLINE long
-conv_rgbF_xyzF (unsigned char *src, unsigned char *dst, long samples)
-{
-  float *src_f = (float *) src;
-  float *dst_f = (float *) dst;
-  long   n     = samples;
-
-  while (n--)
-    {
-      float x, y, z;
-      float red, green, blue;
-
-      red   = src_f[0];
-      green = src_f[1];
-      blue  = src_f[2];
-
-      x = 0.431 * red + 0.342 * green + 0.179 * blue;
-      y = 0.222 * red + 0.707 * green + 0.071 * blue;
-      z = 0.020 * red + 0.130 * green + 0.939 * blue;
-
-      dst_f[0] = x;
-      dst_f[1] = y;
-      dst_f[2] = z;
-      dst_f   += 3;
-      src_f   += 3;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_xyzF_rgbF (unsigned char *src, unsigned char *dst, long samples)
-{
-  float *src_f = (float *) src;
-  float *dst_f = (float *) dst;
-  long   n     = samples;
-
-  while (n--)
-    {
-      dst_f[0] = src_f[0];
-      dst_f[1] = src_f[1];
-      dst_f[2] = src_f[2];
-      dst_f   += 3;
-      src_f   += 3;
-    }
-  return samples;
-}
-
-static INLINE double
-ffunc (const double t)
-{
-  if (t > 0.008856F)
-    {
-      return (cbrt (t));
-    }
-  else
-    {
-      return (7.787F * t + 16.0F / 116.0F);
-    }
-}
-
-
-static INLINE double
-ffunc_inv (const double t)
-{
-  if (t > 0.206893F)
-    {
-      return (t * t * t);
-    }
-  else
-    {
-      return ((t - 16.0F / 116.0F) / 7.787F);
-    }
-}
-
-static INLINE long
-conv_labF_xyzF (unsigned char *src, unsigned char *dst, long samples)
-{
-  float *src_f = (float *) src;
-  float *dst_f = (float *) dst;
-  long   n     = samples;
-
-  while (n--)
-    {
-      float P;
-      float X, Y, Z;
-      float L, a, b;
-
-      L = src_f[0];
-      a = src_f[1];
-      b = src_f[2];
-
-      if (L > LRAMP)
-        {
-          P = Y = (L + 16.0F) / 116.0F;
-          Y = Y * Y * Y;
-        }
-      else
-        {
-          Y = L / 903.3F;
-          P = 7.787F * Y + 16.0F / 116.0F;
-        }
-
-      X = (P + a / 500.0F);
-      X = xnn *ffunc_inv (X);
-      Z = (P - b / 200.0F);
-      Z = znn *ffunc_inv (Z);
-
-
-      dst_f[0] = X;
-      dst_f[1] = Y;
-      dst_f[2] = Z;
-      dst_f   += 3;
-      src_f   += 3;
-    }
-  return samples;
-}
-
-static INLINE long
-conv_xyzF_labF (unsigned char *src, unsigned char *dst, long samples)
-{
-  float *src_f = (float *) src;
-  float *dst_f = (float *) dst;
-  long   n     = samples;
-
-  while (n--)
-    {
-      double ffuncY;
-      float  X, Y, Z;
-      float  L, a, b;
-      X = src_f[0];
-      Y = src_f[1];
-      Z = src_f[2];
-
-      if (Y > 0.0F)
-        {
-          if (Y > 0.008856F)
-            {
-              L = (116.0F * cbrt (Y)) - 16.0F;
-            }
-          else
-            {
-              L = (Y * 903.3F);
-            }
-        }
-      else
-        {
-          L = 0.0;
-        }
-
-      ffuncY = ffunc (Y);
-      a      = 500.0F * (ffunc (X / xnn) - ffuncY);
-      b      = 200.0F * (ffuncY - ffunc (Z / znn));
-
-      dst_f[0] = L;
-      dst_f[1] = a;
-      dst_f[2] = b;
-      dst_f   += 3;
-      src_f   += 3;
-    }
-  return samples;
-}
-
-/******* end of cpercep lift out **/
-
-
-#define MAX_CONVERSIONS    100
-
-typedef struct Conversion
-{
-  int from_fmt;
-  int to_fmt;
-  int cost;               /* cost of function,. calculated as ms to convert 1024*1024 samples */
-  int loss;                     /* 0 maps ok, 1 precision loss, 2 alpha loss, 4 channel loss
-                                   (should be dwelled further into) */
-  void (*function)(unsigned char *src, unsigned char *dst, long samples);
-} Conversion;
-
 int init (void);
 
 int
@@ -1812,29 +1312,11 @@ init (void)
   o (g16, gF);
   o (ga16, ga8);
   o (g16, g8);
-  o (rgbaF, rgbAF);
-  o (rgbAF, rgbaF);
   o (yuvF, rgbF);
   o (yuvaF, rgbaF);
-  o (rgbA8, rgbA16);
-  o (rgb8, rgb16);
   o (ga8, ga16);
   o (gA8, gA16);
   o (g8, g16);
-  o (rgba8, rgbaF);
-  o (rgbA8, rgbAF);
-  o (rgb8, rgbF);
-  o (ga8, gaF);
-  o (gA8, gAF);
-  o (g8, gF);
-  o (rgba16, rgbaF);
-  o (rgbA16, rgbAF);
-  o (rgb16, rgbF);
-  o (ga16, gaF);
-  o (gA16, gAF);
-  o (g16, gF);
-  o (rgbaF, rgbAF);
-  o (rgbAF, rgbaF);
   o (gaF, gAF);
   o (gAF, gaF);
   o (rgbaF, rgbF);
@@ -1847,12 +1329,8 @@ init (void)
   o (gaF, rgbaF);
   o (gAF, rgbAF);
   o (rgbaF, rgb8);
-  o (rgbA8, rgbAF);
-  o (ga8, gaF);
-  o (gA8, gAF);
   o (rgbA8, rgba8);
   o (rgba8, rgbA8);
-  o (gAF, rgbAF);
   o (rgbaF, rgb16);
   o (rgb8, rgba8);
   o (rgb8, rgbA8);
@@ -1860,16 +1338,13 @@ init (void)
   o (rgba8, rgb8);
   o (rgbaF, rgbA8);
   o (rgbA16, rgbaF);
-  o (yuvF, rgbF);
-  o (yuvaF, rgbaF);
   o (rgbaF, rgbaD);
   o (rgbaD, rgbaF);
-  o (rgbaF, rgb8);
-#if 0
-  o (rgbF, xyzF);
-  o (xyzF, rgbF);
-  o (labF, xyzF);
-  o (xyzF, labF);
+  o (rgb8, rgbaF);
+
+#ifdef USE_TABLES
+  if (!table_inited)
+    table_init ();
 #endif
 
   return 0;
